%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Sema Candemir
%
% National Library of Medicine
% National Institutes of Health
% 8600 Rockville Pike
% Bethesda, MD 20894, USA
% Email: sema.candemir@nih.gov
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function CalculateProjectionMatrixForImSimilarity

% addpath('Functions')

ImSource = '.\Model_Xrays\';
imlist = dir([ImSource '*.png']);

NewImageSize = 256;

% NameList = ['12345678';'12345678'];
Model_v = zeros(length(imlist), NewImageSize);
Model_h = zeros(length(imlist), NewImageSize);

disp('Computing X-ray profiles...');

for i = 1:length(imlist)

    [~,ImageName,ext] = fileparts(imlist(i).name);
    % NameList(i,:) = ImageName;
    
    % Read Image
    fname_withPath = sprintf('%s%s',ImSource,imlist(i).name);
    Img = read_Xray(fname_withPath);
     
    [Img crop] = f_removeBlackBorders(Img);  %% crop the black region around the boundary
    Img = double(adapthisteq((Img)));  %% histogram equalization
    % Img = double(histeq((Img)));  %% histogram equalization

    ImHeight = size(Img,1); ImWidth = size(Img,2);
    
    %     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %     %% lung boundary vertical lines.
    %     [spineLine, vProfile] = f_findSpine(ImGray);
    %
    %     [rMask,rlc,rrc,rSlope] = f_findSlopes(spineLine,vProfile); %% take right right col.
    %     [lMask,lc,rc,lSlope] = f_findSlopes(length(vProfile)-spineLine+1,fliplr(vProfile));
    %     llc = length(vProfile)-rc+1; %% take the llc
    %     lrc = length(vProfile)-lc+1;
    %
    % %     imshow(ImGray,[]), hold on
    % %     plot(spineLine,1:ImHeight,'r')
    % %     plot(rrc,1:ImHeight,'r')
    % %     plot(llc,1:ImHeight,'r')
    %
    %     ImGray = ImGray(:,llc:rrc);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % ImGray = imresize(imfilter(ImGray,fspecial('gaussian',7,1.),'same','replicate'),0.125,'bicubic');
    Img = imresize(Img, [NewImageSize NewImageSize],'bilinear');

    % Image Profiles and probabilities
    vProf_Img = sum(Img,1);
    vProf_Img = vProf_Img/sum(vProf_Img);
    % Model_v(i,1) = str2num(ImageName);
    Model_v(i,:) = vProf_Img;
    
    hProf_Img = sum(Img,2);
    hProf_Img = hProf_Img/sum(hProf_Img);
    hProf_Img = hProf_Img';
    % Model_h(i,1) = str2num(ImageName);
    Model_h(i,:) = hProf_Img;
    
end
% save('NameList.mat', 'NameList');
save('Model_v.mat', 'Model_v');
save('Model_h.mat', 'Model_h');



function [spineLine, vProfile] = f_findSpine(inputImage)

vProfile = sum(inputImage,1);

% spine must be in the middle third
mProfile = vProfile;
third = fix(length(vProfile)*0.3);
mProfile(1:third)=0;
mProfile((end-third+1):end)=0;
[~,spineLine] = max(mProfile);

find(mProfile == max(mProfile));


return


function [slopeMask,leftC,rightC,slope] = f_findSlopes(spineColumn,inputImage)

if ((spineColumn < 1) || (spineColumn > size(inputImage,2)))
    fprintf('Spine column in f_findBorders is: %d. \n', spineColumn);
    error('Invalid spine column in f_findBorders.');
end

slopeMask = zeros(size(inputImage));
leftC = ones(size(inputImage,1),1);
rightC = ones(size(inputImage,1),1);
slope = zeros(size(inputImage,1),1);

slopeImage = inputImage(:,spineColumn:end);
localMaxima = zeros(size(slopeImage));
localMinima = zeros(size(slopeImage));

% check boundaries
localMaxima(:,1) = slopeImage(:,1) >= slopeImage(:,2);
localMinima(:,1) = slopeImage(:,1) <= slopeImage(:,2);
localMaxima(:,size(slopeImage,2)) = slopeImage(:,size(slopeImage,2)) >= slopeImage(:,size(slopeImage,2)-1);
localMinima(:,size(slopeImage,2)) = slopeImage(:,size(slopeImage,2)) <= slopeImage(:,size(slopeImage,2)-1);

% process columns
for i=2:(size(slopeImage,2)-1)
    localMaxima(:,i) = (slopeImage(:,i) >= slopeImage(:,i-1)) & (slopeImage(:,i) >= slopeImage(:,i+1));
    localMinima(:,i) = (slopeImage(:,i) <= slopeImage(:,i-1)) & (slopeImage(:,i) <= slopeImage(:,i+1));
end

localMaxima(localMaxima==0) = -realmax;
localMaxima(localMaxima==1) = slopeImage(localMaxima==1);
localMinima(localMinima==0) = realmax;
localMinima(localMinima==1) = slopeImage(localMinima==1);

% search in each row for up-slopes
for i=1:size(slopeImage,1)
    left=0;
    while (left<size(slopeImage,2))
        [pMax,Imax] = max(localMaxima(i,left+1:end),[],2);
        if (pMax==-realmax)
            % no local maximum found (down-slope at end of row)
            break;
        end
        [pMin,Imin] = min(localMinima(i,(left+1):(left+Imax)),[],2);
        if (pMin==realmax)
            % no local minimum found (down-slope at beginning of row)
            left=left+1;
            continue;
        end
        
        pSlope = localMaxima(i,left+Imax)-localMinima(i,left+Imin);
        if (slope(i) < pSlope)
            slope(i) = pSlope;
            leftC(i) = left+Imin;
            rightC(i) = left+Imax;
        end

        left = left+Imax;
    end
end

% adjust indices
leftC = leftC + (spineColumn-1);
rightC = rightC + (spineColumn-1);

% generate slope mask
for i=1:size(slopeMask,1)
    slopeMask(i,leftC(i):rightC(i)) = 1;
end

return





