function maxIndex = FindSimilarCXR(Img,Param)

%-----------------------------------------------------------------------------
%
%   maxIndex = FindSimilarCXR(Img,Param) computes horizontal and vertical
%   profiles of Img, and compares it with the model`s profiles. As
%   similarity metric algorithm uses Bhattacharyya index. The function 
%   returns the indexes of most similar models. 
%
%   Sema Candemir
%   sema.candemir@nih.gov
%   National Library of Medicine, National Institutes of Health
%   8600 Rockville Pike, Bethesda, MD 20894, USA
%
%   References:
%      Candemir, Sema, et al., "Lung segmentation in chest radiographs using 
%      anatomical atlases with nonrigid registration.",
%      Medical Imaging, IEEE Transactions on 33.2 (2014): 577-590.
%
%-----------------------------------------------------------------------------

% Image profiles and probabilities in the query matrix
load('Model_h');
load('Model_v');

% find the most similar images for registration
Img = imresize(Img,[size(Model_h,2),size(Model_h,2)],'bilinear');

% Image Profiles and probabilities of base Image. (the image which we want to segment)
vProf_Img = sum(Img,1);
hProf_Img = sum(Img,2);
vProf_Img = vProf_Img/sum(vProf_Img);
hProf_Img = hProf_Img/sum(hProf_Img);
hProf_Img = hProf_Img';

BC_list = zeros(1,size(Model_v,1));
for i = 1:size(Model_v,1)

    ImgName = Model_v(i,1);
    vProf_ImTemp = Model_v(i,:);
    hProf_ImTemp = Model_h(i,:);
    
    % Bhattacharyya coefficient
    BC_v = sum(sqrt(vProf_Img.*vProf_ImTemp));
    BC_h = sum(sqrt(hProf_Img.*hProf_ImTemp));
    BC = (BC_v + BC_h)/2;
    BC_list(1,i) = ImgName;
    BC_list(2,i) = BC;
end
[~,sortIndex] = sort(BC_list(2,:),'descend');

% choose the top K similar image
maxIndex = sortIndex(1:Param.NofMask);  
