function [spcurve] = cebShape2Spline(A)
%
% CEBSHAPE2SPLINE
%	[spcurve] = cebShape2Spline(A)
%
%	This function returns the spline points in an array after
%	computing the spline curve from the input shape points
%
% 	A type of data interpolation
%
% Input:
%		A			Mx2 matrix organized with x in column 1
%					y in column 2
%	
%					[ 	x1 y1;
%						x2 y2;
%						...
%						xM yM	];
%
%	Output:
%		spcurve		The spline points calculated from the shape points
%					Mx2 matrix organized with x in column 1
%					y in column 2
%	
%					[ 	x1 y1;
%						x2 y2;
%						...
%						xN yN	];

% Created by:		Dan Krainak
% Creation date:	25-Jul-2002
% 
% Version:			1.0
% Version date: 	25-Jul-2002
%
% Modified from CEB_ReadShapeFile
% Created by:           Sameer Antani
% Creation Date:        June 12, 2002

[r,c] = size(A);                    % Get Matrix Dimensions

% insure proper size
if (c == 2)

	A(r+1,:) = A(1,:);                  % Add one more point to close the curve (first=last)

	% create knots in appropriate format
	curvepoints = [A(:,1)'; A(:,2)'];

	% fit spline
	curve = fnplt(cscvn(curvepoints));

	% transpose matrix
	spcurve = curve(:,:)';          % Use spline output
else
	disp('Dk0945: incorrect input size, not Mx2 matrices');
end