function NewMask = f_RemoveSmallSegments(Mask)

%-----------------------------------------------------------------------------
%
%   NewMask = f_RemoveSmallSegments(Mask) reduces the noise on the mask
%   occured during warping. The function assumes that two largest blobs 
%   correspond to lung masks, and ignore all other small blobs. 
%
%   Sema Candemir
%   sema.candemir@nih.gov
%   National Library of Medicine, National Institutes of Health
%   8600 Rockville Pike, Bethesda, MD 20894, USA

%   Reference:
%      Candemir, Sema, et al., "Lung segmentation in chest radiographs using 
%      anatomical atlases with nonrigid registration.",
%      Medical Imaging, IEEE Transactions on 33.2 (2014): 577-590.
%
%-----------------------------------------------------------------------------

NewMask = zeros(size(Mask,1),size(Mask,2));

% remove the small segments.
ConnCompInfo = bwconncomp(Mask);
numPixels = cellfun(@numel,ConnCompInfo.PixelIdxList);
[biggest,idx] = max(numPixels);
Mask(ConnCompInfo.PixelIdxList{idx}) = 0;
NewMask(ConnCompInfo.PixelIdxList{idx}) = 1;

ConnCompInfo = bwconncomp(Mask);
numPixels = cellfun(@numel,ConnCompInfo.PixelIdxList);
[biggest,idx] = max(numPixels);
NewMask(ConnCompInfo.PixelIdxList{idx}) = 1;

NewMask = imfill(NewMask);

