%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Dr. Stefan Jaeger (c) 2011
%
% National Library of Medicine
% National Institutes of Health
% 8600 Rockville Pike
% Bethesda, MD 20894, USA
% Email: stefan.jaeger@nih.gov
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Remove black borders from image
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% [outputImage] = f_getLungBoxCoordinates(inputImage)
%
% Input:
% inputImage, two-dimensional input Image
%
% Output:
% outputImage, output Image
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [outputImage, crop] = f_removeBlackBorders(inputImage)

if (ndims(inputImage)>2)
    error('f_removeBlackBorders does not support more than two input dimensions.');
end

% remove top and bottom borders
rowSums = sum(inputImage,2);
ur = find(rowSums>0,1,'first');
lr = find(rowSums>0,1,'last');

% remove left and right borders
colSums = sum(inputImage,1);
lc = find(colSums>0,1,'first');
rc = find(colSums>0,1,'last');

% calculte the crop pixels
crop.ur = 0;
crop.lr = 0;
crop.lc = 0;
crop.rc = 0;
if(ur ~= 1) crop.ur = ur-1; end
if(lr ~= size(inputImage,1)) crop.lr = size(inputImage,1)-lr; end
if(lc ~= 1) crop.lc = lc-1; end
if(rc ~= size(inputImage,2)) crop.rc = size(inputImage,2)-rc; end 

if (~isempty(ur) && ~isempty(lc))
    outputImage = inputImage(ur:lr,lc:rc);
else
    % return black input image
    outputImage = inputImage;
end

return